Package: fitdistrplus
Title: Help to Fit of a Parametric Distribution to Non-Censored or
        Censored Data
Version: 1.2-1
Authors@R: c(person("Marie-Laure", "Delignette-Muller", role = "aut", email = "marielaure.delignettemuller@vetagro-sup.fr", comment = c(ORCID = "0000-0001-5453-3994")),
             person("Christophe", "Dutang", role = "aut", email = "christophe.dutang@ensimag.fr", comment = c(ORCID = "0000-0001-6732-1501")),
             person("Regis", "Pouillot", role = "ctb"),
             person("Jean-Baptiste", "Denis", role = "ctb"),
             person("Aurélie", "Siberchicot", role = c("aut", "cre"), email = "aurelie.siberchicot@univ-lyon1.fr", comment = c(ORCID = "0000-0002-7638-8318")))
Description: Extends the fitdistr() function (of the MASS package) with several functions 
  to help the fit of a parametric distribution to non-censored or censored data. 
  Censored data may contain left censored, right censored and interval censored values, 
  with several lower and upper bounds. In addition to maximum likelihood estimation (MLE), 
  the package provides moment matching (MME), quantile matching (QME), maximum goodness-of-fit 
  estimation (MGE) and maximum spacing estimation (MSE) methods (available only for 
  non-censored data). Weighted versions of MLE, MME, QME and MSE are available. See e.g. 
  Casella & Berger (2002), Statistical inference, Pacific Grove, for a general introduction 
  to parametric estimation.
Depends: R (>= 3.5.0), MASS, grDevices, survival, methods
Imports: stats, rlang
Suggests: actuar, rgenoud, mc2d, gamlss.dist, knitr, ggplot2,
        GeneralizedHyperbolic, rmarkdown, Hmisc, bookdown
VignetteBuilder: knitr
BuildVignettes: true
License: GPL (>= 2)
Encoding: UTF-8
URL: https://lbbe-software.github.io/fitdistrplus/,
        https://lbbe.univ-lyon1.fr/fr/fitdistrplus,
        https://github.com/lbbe-software/fitdistrplus
BugReports: https://github.com/lbbe-software/fitdistrplus/issues
Contact: Marie-Laure Delignette-Muller
        <marielaure.delignettemuller@vetagro-sup.fr> or Christophe
        Dutang <christophe.dutang@ensimag.fr>
NeedsCompilation: no
Packaged: 2024-07-11 12:47:51 UTC; siberchicot
Author: Marie-Laure Delignette-Muller [aut]
    (<https://orcid.org/0000-0001-5453-3994>),
  Christophe Dutang [aut] (<https://orcid.org/0000-0001-6732-1501>),
  Regis Pouillot [ctb],
  Jean-Baptiste Denis [ctb],
  Aurélie Siberchicot [aut, cre]
    (<https://orcid.org/0000-0002-7638-8318>)
Maintainer: Aurélie Siberchicot <aurelie.siberchicot@univ-lyon1.fr>
Repository: CRAN
Date/Publication: 2024-07-12 12:20:02 UTC
Built: R 4.4.1; ; 2024-07-16 08:56:48 UTC; unix
